/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/


#include <pelib/AmplOutputData.hpp>
#include <pelib/DataParser.hpp>
#include <pelib/Scalar.hpp>
#include <pelib/CastException.hpp>
#include <pelib/ParseException.hpp>
#include <pelib/NoDecimalFloatException.hpp>

#ifndef PELIB_AMPLOUTPUTSCALAR
#define PELIB_AMPLOUTPUTSCALAR

namespace pelib
{
	template <class Value>
	class
	AmplOutputScalar: public AmplOutputData
	{
		public:
			AmplOutputScalar(bool strict = true)
			{
				this->strict = strict;
			}
			
			virtual
			AmplOutputScalar*
			clone() const
			{
				return new AmplOutputScalar();
			}

			virtual
			std::string
			getDetailedPattern()
			{
				return "(\\w[\\w\\d_]*)\\s*=\\s*([\\w\\d][\\w\\d_\\.+]*)";
			}

			virtual
			std::string
			getGlobalPattern()
			{
				return "\\w[\\w\\d_]*\\s*=\\s*[\\w\\d][\\w\\d_\\.+]*";
			}

			virtual
			AlgebraData*
			parse(std::istream &in)
			{
				std::string str((std::istreambuf_iterator<char>(in)), std::istreambuf_iterator<char>());

				boost::cmatch match;
				try
				{
					match = AlgebraDataParser::match(std::string("(?:.*?)").append(getDetailedPattern()), str);
				} catch(NoDecimalFloatException &e)
				{
					std::ostringstream ss;
					ss << e.getValue();
					throw ParseException(std::string("Asked a decimal conversion, but \"").append(ss.str()).append("\" is integer."));
				}

				Scalar<Value> *scalar = new Scalar<Value>(match[1], AlgebraDataParser::convert<Value>(match[2], strict));
				return scalar;
			}

			virtual
			void
			dump(std::ostream &stream, const AlgebraData *data) const
			{
				const Scalar<Value> *scalar = dynamic_cast<const Scalar<Value>* >(data);
				if(scalar == NULL) throw CastException("parameter \"data\" was not of type \"Scalar<Value>\".");

				stream << scalar->getName() << " = " << scalar->getValue() << std::endl;
			}
	
		protected:
			bool strict;
		private:		
	};
}

#endif
